package com.alicloud.openservices.tablestore.core.protocol;

import com.alicloud.openservices.tablestore.ClientException;
import com.alicloud.openservices.tablestore.model.*;
import com.alicloud.openservices.tablestore.model.internal.TableOptionsEx;

import java.io.IOException;
import java.util.List;

public class OTSProtocolParser {
    public static PrimaryKeyType toPrimaryKeyType(OtsInternalApi.PrimaryKeyType type) {
        switch (type) {
            case INTEGER:
                return PrimaryKeyType.INTEGER;
            case STRING:
                return PrimaryKeyType.STRING;
            case BINARY:
                return PrimaryKeyType.BINARY;
            default:
                throw new IllegalArgumentException("Unknown primary key type: " + type);
        }
    }

    public static TableMeta parseTableMeta(OtsInternalApi.TableMeta tableMeta) {
        TableMeta result = new TableMeta(tableMeta.getTableName());
        for (OtsInternalApi.PrimaryKeySchema pk : tableMeta.getPrimaryKeyList()) {
            result.addPrimaryKeyColumn(pk.getName(), toPrimaryKeyType(pk.getType()));
        }
        return result;
    }

    public static ReservedThroughputDetails parseReservedThroughputDetails(
            OtsInternalApi.ReservedThroughputDetails reservedThroughputDetails) {
        return new ReservedThroughputDetails(
            parseCapacityUnit(reservedThroughputDetails.getCapacityUnit()),
            reservedThroughputDetails.getLastIncreaseTime(),
            reservedThroughputDetails.getLastDecreaseTime());
    }

    public static CapacityUnit parseCapacityUnit(OtsInternalApi.CapacityUnit capacityUnit) {
        CapacityUnit result = new CapacityUnit();
        if (capacityUnit.hasRead()) {
            result.setReadCapacityUnit(capacityUnit.getRead());
        }

        if (capacityUnit.hasWrite()) {
            result.setWriteCapacityUnit(capacityUnit.getWrite());
        }
        return result;
    }

    public static TableOptionsEx
        parseTableOptions(OtsInternalApi.TableOptions tableOptions) {
        TableOptionsEx result =
            new TableOptionsEx();

        if (tableOptions.hasBloomFilterType()) {
            result.setBloomFilterType(toBloomFilterType(tableOptions.getBloomFilterType()));
        }

        if (tableOptions.hasDeviationCellVersionInSec()) {
            result.setMaxTimeDeviation(tableOptions.getDeviationCellVersionInSec());
        }

        if (tableOptions.hasMaxVersions()) {
            result.setMaxVersions(tableOptions.getMaxVersions());
        }

        if (tableOptions.hasTimeToLive()) {
            result.setTimeToLive(tableOptions.getTimeToLive());
        }

        if (tableOptions.hasBlockSize()) {
            result.setBlockSize(tableOptions.getBlockSize());
        }

        return result;
    }

    private static com.alicloud.openservices.tablestore.model.internal.BloomFilterType
        toBloomFilterType(OtsInternalApi.BloomFilterType bloomFilterType) {
        switch (bloomFilterType) {
            case NONE:
                return com.alicloud.openservices.tablestore.model.internal.BloomFilterType.NONE;
            case CELL:
                return com.alicloud.openservices.tablestore.model.internal.BloomFilterType.CELL;
            case ROW:
                return com.alicloud.openservices.tablestore.model.internal.BloomFilterType.ROW;
            default:
                throw new ClientException("Unknown bloom filter type:" + bloomFilterType);
        }
    }

    public static BatchGetRowResponse.RowResult parseBatchGetRowStatus(String tableName, OtsInternalApi.RowInBatchGetRowResponse status, int index) {
        if (status.getIsOk()) {
            Row row = null;
            if (!status.getRow().isEmpty()) {
                try {
                    PlainBufferCodedInputStream inputStream = new PlainBufferCodedInputStream(new PlainBufferInputStream(status.getRow().asReadOnlyByteBuffer()));
                    List<PlainBufferRow> rows = inputStream.readRowsWithHeader();
                    if (rows.size() != 1) {
                        throw new IOException("Expect only returns one row. Row count: " + rows.size());
                    }
                    row = PlainBufferConversion.toRow(rows.get(0));
                } catch (Exception e) {
                    throw new ClientException("Failed to parse row data.", e);
                }
            }
            ConsumedCapacity consumedCapacity = new ConsumedCapacity(parseCapacityUnit(status.getConsumed().getCapacityUnit()));
            if (status.hasNextToken()) {
                return new BatchGetRowResponse.RowResult(tableName, row, consumedCapacity, index, status.getNextToken().toByteArray());
            } else {
                return new BatchGetRowResponse.RowResult(tableName, row, consumedCapacity, index);
            }
        } else {
        	com.alicloud.openservices.tablestore.model.Error error = new com.alicloud.openservices.tablestore.model.Error(status.getError().getCode(), status.getError().getMessage());
            return new BatchGetRowResponse.RowResult(tableName, error, index);
        }
    }

    public static BatchWriteRowResponse.RowResult parseBatchWriteRowStatus(String tableName, OtsInternalApi.RowInBatchWriteRowResponse status, int index) {
        if (status.getIsOk()) {
            ConsumedCapacity consumedCapacity = new ConsumedCapacity(parseCapacityUnit(status.getConsumed().getCapacityUnit()));
            return new BatchWriteRowResponse.RowResult(tableName, consumedCapacity, index);
        } else {
        	com.alicloud.openservices.tablestore.model.Error error = new com.alicloud.openservices.tablestore.model.Error(status.getError().getCode(), status.getError().getMessage());
            return new BatchWriteRowResponse.RowResult(tableName, error, index);
        }
    }
}
